#include "HrbPlot.h"
#include "ui_HrbPlot.h"

#include <qwt_plot.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_marker.h>
#include <qwt_interval_data.h>
#include <qwt_valuelist.h>

#include "qwt_plot.h"
#include "qwt_plot_grid.h"
#include "qwt_interval_data.h"
#include "qwt_scale_engine.h"

#include "HistogramItem.h"
#include <QThread>
#include "TcpClient.h"
#include "QSemaphore"
#include <QSettings>
#include <QHostAddress>
#include <QShortcut>
#include <QFileDialog>

HrbPlot::HrbPlot(quint32 serialId, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::HrbPlot),
    serial(serialId),
    chPlot(new QwtPlot),
    samPlot(new QwtPlot)
{
    ui->setupUi(this);
    ui->groupBoxDetail->setTitle(QString("%1-%2")
                                 .arg(serial>>16,4,16,QChar('0'))
                                 .arg(serial&0xFFFF,4,16,QChar('0')));
    gropingUpdated(TYPE_CH);

    setupPlot(chPlot);
    chPlot->setAxisScale(QwtPlot::xBottom, 0, 96, 16);
//    chPlot->setAxisTitle(QwtPlot::xBottom, "channel number");

    setupPlot(samPlot);
//    samPlot->setAxisTitle(QwtPlot::xBottom, "sample, ns");

    ui->verticalLayout_3->addWidget(chPlot);
    ui->verticalLayout_3->addWidget(samPlot);
    samPlot->hide();
}

HrbPlot::~HrbPlot()
{
    delete ui;
}
namespace  {
const int HRB_CH_NUMBER = 96;
}

void HrbPlot::updateCounters(QVector<QVector<quint64> > data, quint32 trigPassed, double histLimit)
{
    QVector<double> ch(HRB_CH_NUMBER, 0);
    QVector<double> samples;
    double integral = 0;
    int samNumber = data.size();
    samples.resize(data.size());
    if(samNumber){
        ch.resize(data[0].size());
    }
    for(int samId=0; samId<samNumber; ++samId){
        for(int chId=0; chId<data[samId].size(); ++chId){
            quint64 &val = data[samId][chId];
            ch[chId] += val;
            samples[samId] += val;
            if(histLimit){
                ch[chId] = qMin(ch[chId], histLimit);
                samples[samId] = qMin(samples[samId], histLimit);
            }
        }
    }

    foreach (double v, samples)
        integral += v;

    QwtScaleEngine *scaleEngine = new QwtLinearScaleEngine();
    *scaleEngine = *chPlot->axisScaleEngine(QwtPlot::xBottom);
    scaleEngine->setAttribute(QwtScaleEngine::Inverted);

    replot(chPlot, ch, 1);
    replot(samPlot, samples, 8);

    ui->labelPassedTrig->setText(QString::number(trigPassed));
    ui->labelPassedTrig->setText(QString::number(trigPassed));
    ui->labelPassedTrig->setText(QString::number(trigPassed));

    ui->labelIntegral->setText(QString::number(integral,'f',0));
    ui->labelPassedTrig->setText(QString::number(trigPassed));
    ui->labelRate->setText(QString("%1").arg(trigPassed?integral/trigPassed:0, 2,'g', 2));

//    //    emit connectedStateChanged(index, online);

//        int sum=0;
//    //    QwtPlot *plot = ui->plot;
//        QwtPlot *plot = new QwtPlot();

//        HistogramItem *histogram;
//        foreach(QwtPlotItem *item, plot->itemList()){
//            histogram = dynamic_cast<HistogramItem*>(item);
//            if(histogram) break;
//        }

//        if(!histogram){
//            histogram = new HistogramItem();
//            histogram->setColor(Qt::darkCyan);
//            histogram->attach(plot);
//        }

//        int numValues = 0;
//        bool flagOk = false;
////        quint32 devId = curHrbIds.toUInt(&flagOk, 16);
//        quint32 devId = curHrbIds;
//        if(!flagOk){
//            devId=0;
//        } else {
//    //        numValues = reader->getCounters()[devId].size(); //todo
//        }

//        QwtArray<QwtDoubleInterval> intervals(numValues);
//        QwtArray<double> values(numValues);

//        int pos = 0;
//        for (int i = 0; i < numValues; ++i)
//        {
//            intervals[i] = QwtDoubleInterval(pos, pos + 1);
//    //        values[i] = reader->getCounters()[devId][i];//todo
//    //        sum += reader->getCounters()[devId][i];//todo
//            ++pos;
//        }

//        histogram->setData(QwtIntervalData(intervals, values));

//        qDebug() << "Print " << curHrbIds;
//        plot->replot();
//        //todo
//    //    ui->labelSum->setText(QString("Integral: %1; Passed Trig: %2; Rate: %3")
//    //                          .arg(sum)
//    //                          .arg(reader->getPassTrig())
//    //                          .arg(reader->getPassTrig()?(double)sum/reader->getPassTrig():0, 2,'g', 2));
}

void HrbPlot::initialize()
{

}

void HrbPlot::updateHistogram(int index, quint32 devId, QVector<QVector<quint64> > cur, quint32 trig)
{

}

void HrbPlot::updateStatus(bool status)
{

}

void HrbPlot::gropingUpdated(bool isByChannels)
{
    isGroppingByChannels = isByChannels;
    chPlot->hide();
    samPlot->hide();
    chPlot->setHidden(!isGroppingByChannels);
    samPlot->setHidden(isGroppingByChannels);
}

void HrbPlot::gropingUpdated(int type)
{
    chPlot->hide();
    samPlot->hide();
    ui->groupBoxDetail->hide();
    chPlot->setHidden(type != TYPE_CH);
    samPlot->setHidden(type != TYPE_SAMPLE);
    ui->groupBoxDetail->setHidden(type != TYPE_INFO);
}

void HrbPlot::setupPlot(QwtPlot *plot)
{
    plot->setCanvasBackground(QColor(Qt::white));

    QwtPlotGrid *grid = new QwtPlotGrid(); //todo should here mem leak?
    grid->enableXMin(true);
    grid->enableYMin(true);
//    grid->setMajPen(QPen(Qt::black, 0, Qt::DotLine));
//    grid->setMinPen(QPen(Qt::gray, 0 , Qt::DotLine));
    grid->setMajPen(QPen(Qt::black, 0, Qt::SolidLine));
    grid->setMinPen(QPen(Qt::gray, 0 , Qt::DotLine));
    grid->attach(plot);

    QString serialStr = QString("%1-%2")
            .arg(serial >> 16,4,16,QChar('0'))
            .arg(serial & 0xFFFFu,4,16,QChar('0'));

    plot->setTitle(serialStr);
    plot->setToolTip(serialStr);

    QwtText title = plot->title();
    QFont titleFont = title.font();
    titleFont.setPointSize(8);
    title.setFont(titleFont);
    plot->setTitle(title);

    int minWeight=100, minHeight=100;
    plot->setMinimumSize(minWeight,minHeight);
//    plot->resize(minHeight,minHeight);

    plot->setFrameStyle(QFrame::Panel | QFrame::Raised);
    plot->setLineWidth(2);
}


void HrbPlot::replot(QwtPlot *plot, QVector<double> data, int xStep)
{
    //    qDebug()<<plot->axisScaleEngine(QwtPlot::xBottom)->attributes();
        HistogramItem *histogram;
        foreach(QwtPlotItem *item, plot->itemList()){
            histogram = dynamic_cast<HistogramItem*>(item);
            if(histogram) break;
        }

        if(!histogram){
            histogram = new HistogramItem();
            histogram->setColor(Qt::darkCyan);
            histogram->attach(plot);
        }

        const int numValues = data.size();

        QwtArray<QwtDoubleInterval> intervals(numValues);
        QwtArray<double> values(numValues);

        int pos = 0;
        for (int i = 0; i < numValues; ++i)
        {
            intervals[i] = QwtDoubleInterval(pos*xStep, (pos + 1)*xStep);
            values[i] = data[i];
            ++pos;
        }

        histogram->setData(QwtIntervalData(intervals, values));

        plot->replot();
}
