#include "HrbAppConfigConverter.h"

#include "HrbAppConfig.h"
#include "base/ConfigConverterUtil.h"


namespace {
constexpr const char *k_defaultHrb = "defaultHrb";
constexpr const char *k_hrb = "hrb";
constexpr const char *k_ttb = "ttb";
constexpr const char *k_allowRemoteControl = "allowRemoteControl";
constexpr const char *k_showAveragedBusy = "showAveragedBusy";
constexpr const char *k_triggerSetup = "triggerSetup";

//HrbTriggerSetup
constexpr const char *k_ttlSpill = "ttlSpill";
constexpr const char *k_ttlTrigger = "ttlTrigger";
constexpr const char *k_lvdsSpill = "lvdsSpill";
constexpr const char *k_lvdsTrigger = "lvdsTrigger";
constexpr const char *k_internalSpill = "internalSpill";
constexpr const char *k_internalTrigger = "internalTrigger";
constexpr const char *k_interval = "interval";
constexpr const char *k_softSpill = "softSpill";
} // namespace

//  ==== HrbAppConfig start ====
template<>
bool AbstractConfigConverter<HrbAppConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<HrbAppConfig>::toJsonObject(const HrbAppConfig &in)
{
    QVariantMap m;
    m[k_defaultHrb] = QVariant::fromValue(in.defaultHrb);
    m[k_hrb] = ConfigConverterUtil::toJsonValue(in.hrb);
    m[k_ttb] = ConfigConverterUtil::toJsonValue(in.ttb);
    m[k_triggerSetup] = QVariant::fromValue(in.triggerSetup);
    m[k_allowRemoteControl] = QVariant::fromValue(in.allowRemoteControl);
    m[k_showAveragedBusy] = QVariant::fromValue(in.showAveragedBusy);
    return fromVariantMap(m);
}

template<>
HrbAppConfig AbstractConfigConverter<HrbAppConfig>::fromJsonObject(const QJsonObject &in)
{
    HrbAppConfig out;
    update_value(in, k_defaultHrb, &out.defaultHrb);
    out.hrb = ConfigConverterUtil::toMap<DeviceIndex, ModularDeviceConfig>(in[k_hrb]);
    out.ttb = ConfigConverterUtil::toMap<DeviceIndex, ModularDeviceConfig>(in[k_ttb]);
    update_value(in, k_triggerSetup, &out.triggerSetup);
    update_value(in, k_allowRemoteControl, &out.allowRemoteControl);
    update_value(in, k_showAveragedBusy, &out.showAveragedBusy);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<HrbAppConfig>::toJsonValue(const HrbAppConfig &in)
{
    return toJsonObject(in);
}

template<>
HrbAppConfig AbstractConfigConverter<HrbAppConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<HrbAppConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<ModularDeviceConfig>::registerTypes();
    AbstractConfigConverter<HrbTriggerSetup>::registerTypes();
}
//  ==== HrbAppConfig end   ====

//  ==== HrbTriggerSetup start ====
template<>
bool AbstractConfigConverter<HrbTriggerSetup>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<HrbTriggerSetup>::toJsonObject(const HrbTriggerSetup &in)
{
    QVariantMap m;
    m[k_ttlSpill] = QVariant::fromValue(in.ttlSpill);
    m[k_ttlTrigger] = QVariant::fromValue(in.ttlTrigger);
    m[k_lvdsSpill] = QVariant::fromValue(in.lvdsSpill);
    m[k_lvdsTrigger] = QVariant::fromValue(in.lvdsTrigger);
    m[k_internalSpill] = QVariant::fromValue(in.internalSpill);
    m[k_internalTrigger] = QVariant::fromValue(in.internalTrigger);
    m[k_interval] = QVariant::fromValue(in.interval);
    m[k_softSpill] = QVariant::fromValue(in.softSpill);
    return fromVariantMap(m);
}

template<>
HrbTriggerSetup AbstractConfigConverter<HrbTriggerSetup>::fromJsonObject(const QJsonObject &in)
{
    HrbTriggerSetup out;
    update_value(in, k_ttlSpill, &out.ttlSpill);
    update_value(in, k_ttlTrigger, &out.ttlTrigger);
    update_value(in, k_lvdsSpill, &out.lvdsSpill);
    update_value(in, k_lvdsTrigger, &out.lvdsTrigger);
    update_value(in, k_internalSpill, &out.internalSpill);
    update_value(in, k_internalTrigger, &out.internalTrigger);
    update_value(in, k_interval, &out.interval);
    update_value(in, k_softSpill, &out.softSpill);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<HrbTriggerSetup>::toJsonValue(const HrbTriggerSetup &in)
{
    return toJsonObject(in);
}

template<>
HrbTriggerSetup AbstractConfigConverter<HrbTriggerSetup>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<HrbTriggerSetup>::registerSubTypes()
{}
//  ==== HrbTriggerSetup end   ====
