#include "TtbDeviceGlobal.h"

#include "TtbDevice.h"

TtbDeviceGlobal::TtbDeviceGlobal(const DeviceIndex &index, QObject *parent)
    :
    MstreamMregDevGlobal(new TtbDevice(index), devStatus, parent)
{
}

void TtbDeviceGlobal::deviceStatusUpdated(const TtbStatus &status)
{
    Q_ASSERT(getDeviceIndex() == status.getDeviceIndex());
    if(getDeviceIndex() != status.getDeviceIndex())
        return;

    if(inRun) {
        TtbStatus diff = status;
        diff -= devStatus;
        isTtbStatusOk(diff);
    }
    devStatus = status;
}

bool TtbDeviceGlobal::isTtbStatusOk(const TtbStatus &status, bool silent) const
{
    QString errWhat;
    for(int ch=0;ch<TTB_CH_COUNT;++ch) {
        if(status.statistic.linkUp[ch])
            errWhat.append(QString("LinkUp[ch:%1]=%2; ").arg(ch).arg(status.statistic.linkUp[ch]));
        if(status.statistic.linkError[ch])
            errWhat.append(QString("LinkError[ch:%1]=%2; ").arg(ch).arg(status.statistic.linkError[ch]));
        if(status.statistic.linkTimeout[ch])
            errWhat.append(QString("LinkTimeout[ch:%1]=%2; ").arg(ch).arg(status.statistic.linkTimeout[ch]));
        if(status.statistic.linkErrDisable[ch])
            errWhat.append(QString("LinkErrDisable[ch:%1]=%2; ").arg(ch).arg(status.statistic.linkErrDisable[ch]));

        if(status.statistic.linkErrDisableBusySet[ch])
            errWhat.append(QString("linkErrDisableBusySet[ch:%1]=%2; ").arg(ch).arg(status.statistic.linkErrDisableBusySet[ch]));
        if(status.statistic.linkErrDisableBusyUnset[ch])
            errWhat.append(QString("linkErrDisableBusyUnset[ch:%1]=%2; ").arg(ch).arg(status.statistic.linkErrDisableBusyUnset[ch]));
        if(status.statistic.linkErrDisableBusyMStream[ch])
            errWhat.append(QString("linkErrDisableBusyMStream[ch:%1]=%2; ").arg(ch).arg(status.statistic.linkErrDisableBusyMStream[ch]));
    }
    bool ok = errWhat.isNull();
    if(!ok && !silent){
        const QString &errStr = QString("[%1] Error in status: %2")
                .arg(status.getDeviceIdent())
                .arg(errWhat);
        qWarning() << errStr;
        emit runError(getDeviceIndex(), errStr);
    }
    return ok;
}
