#include "JsonViewer.h"
#include "Viewer.h"
#include "base/DaqModule.h"
#include "base/ConfigIndexName.h"
#include "base/ConfigSelector.h"
#include "base/ConfigDocument.h"
#include "base/RcRunId.h"
#include "mongo/RootConfigUtil.h"
#include "mongo/JsonDB.cpp"

#include <QStringList>
#include <QDir>
#include <QLabel>

bool jsonLessThan(const QJsonObject &o1, const QJsonObject &o2)
{
    QDateTime modification_dt_1 = RootConfigUtil::getModificationTimestamp(o1);
    QDateTime modification_dt_2 = RootConfigUtil::getModificationTimestamp(o2);
    return modification_dt_1 < modification_dt_2;
}

JsonViewer::JsonViewer()
{}

QList<ConfigDocument> JsonViewer::aggregate(const MongoViewerConfig &config)
{
    QList<ConfigDocument> to_return;
    qInfo() << "=== JsonViewer::aggregate " << get_application_path();
    QDir directory(get_application_path());
    QDirIterator it(get_application_path(), {"*.json"}, QDir::NoFilter, QDirIterator::Subdirectories);
    while (it.hasNext()) {
        QFile config_file(it.next());
        if(!config_file.open(QIODevice::ReadOnly)){
            continue;
        }
        QJsonParseError json_err;
        QByteArray data = config_file.readAll();
        QJsonDocument json_doc(QJsonDocument::fromJson(data, &json_err));
        if (QJsonParseError::NoError != json_err.error) {
            continue;
        }
        QJsonObject obj = json_doc.object();
        to_return.append(QVariant::fromValue(obj).value<ConfigDocument>());
    }
    return to_return;
}

QList<QJsonObject> JsonViewer::agregate_json(const MongoViewerConfig &config)
{
    QList<QJsonObject> to_return = agregate_cs_json(config).values();

    if(config.type == MongoViewerConfig::HISTORY){
        QList<RcRunId> rc_run_id_list = get_rc_run_ids(config);
//        for(int i=0; i<rc_run_id_list.size(); ++i){
//            qInfo() << "=== " << rc_run_id_list[i].toString();
//        }
    }
//        QList<DaqModule> all_modules = DaqModule::get_all_modules();
//        for(int i=0; i<all_modules.size(); ++i){
//            DaqModule dm = all_modules[i];
//            QMap<QString, QStringList> config_map = get_config_map(all_modules[i]);
//            for(QString program_index : config_map.keys()){
//                for(int j=0; j<config_map[program_index].size(); ++j){
//                    if(!config.program_type_filter.isEmpty())
//                        if(dm.toString() != config.program_type_filter)
//                            continue;
//                    if(!config.program_index_filter.isEmpty())
//                        if(program_index != config.program_type_filter)
//                            continue;
//                    ConfigurationName cn(config_map[program_index][j]);
//                    ProgramIndex pi(program_index);
//                    ConfigIndexName ci(pi,cn);
//                    ConfigSelector cs(dm, ci);
//                    QJsonObject obj = read_config(cs);
//                    to_return.append(obj);
//                    qInfo() << "=== 1";
////                    QDateTime modification_dt = RootConfigUtil::getModificationTimestamp(obj);
//                }
//            }
//        }
//        std::sort(to_return.begin(), to_return.end(), jsonLessThan);
//        int pos = (config.page_number - 1) * config.rows_on_page + 1;
//        to_return.mid(pos, config.rows_on_page);

    return to_return;
}




QMap<ConfigSelector, QJsonObject> JsonViewer::agregate_cs_json(const MongoViewerConfig &config)
{
    QMap<ConfigSelector, QJsonObject> to_return;

    if(config.type == MongoViewerConfig::SAVED){
        QList<ConfigSelector> cs_list = get_config_selectors(config);
        for(int i=0; i<cs_list.size(); ++i){
            ConfigSelector cs = cs_list[i];
            if(!config.program_type_filter.isEmpty())
                if(cs.getType().toString() != config.program_type_filter)
                    continue;
            if(!config.program_index_filter.isEmpty())
                if(cs.getIndex().toString() != config.program_index_filter)
                    continue;
            auto obj = QVariant::fromValue(readConfigDocument(cs)).value<QJsonObject>();
            to_return.insert(cs, obj);
        }
    }
    return to_return;
}

QMap<RcRunId, QJsonObject> JsonViewer::agregate_ri_json(const MongoViewerConfig &config)
{
    return QMap<RcRunId, QJsonObject>();
}

QList<ConfigSelector> JsonViewer::get_config_selectors(const MongoViewerConfig &)
{
    QList<ConfigSelector> to_return;
    auto map = JsonDB::get_config_map(DaqModule(DaqModule::Type::TQDC));
//    for(auto pi : map.keys()){
//        for(auto cn : map[pi])
//            to_return << ConfigSelector(DaqModule(DaqModule::Type::TQDC),
//                                        ProgramIndex(pi),
//                                        ConfigurationName(cn));
//    }


//    QString application_path = JsonDBUtil::get_application_path();
//    QList<QFileInfo> types = QDir(application_path)
//        .entryInfoList(QDir::NoDotAndDotDot|QDir::Dirs);
//    for(int i=0; i<types.size(); ++i){
//        QString program_type = types[i].baseName();
//        DaqModule module_tmp = DaqModule::fromString(program_type);
//        if(module_tmp.isEmpty())
//            continue;
//        QString type_path = application_path + "/" + program_type;
//        QList<QFileInfo> indexes = QDir(type_path)
//            .entryInfoList(QDir::NoDotAndDotDot|QDir::Dirs);
//        for(int j=0; j<indexes.size(); ++j){
//            QString program_index = indexes[j].baseName();
//            QString index_path = type_path + "/" + program_index;
//            QList<QFileInfo> configs = QDir(index_path)
//                .entryInfoList(QDir::NoDotAndDotDot|QDir::Files);
//            for(int k=0; k<configs.size(); ++k){
//                QString config_name = configs[k].baseName();
//                DaqModule dm;
//                dm = DaqModule::fromString(program_type);
//                qInfo() <<  dm.toString() << "   " << program_type;
//                ProgramIndex pi(program_index);
//                ConfigurationName ci(config_name);
//                ConfigIndexName cin(pi, ci);
//                ConfigSelector cs(dm, cin);
//                to_return.append(cs);
//            }
//        }
//    }
    return to_return;
}

QStringList JsonViewer::get_program_types(const MongoViewerConfig &config)
{
    // TODO
    return {};
}

QStringList JsonViewer::get_program_indexes(const MongoViewerConfig &config)
{
    // TODO
    return {};
}

QList<RcRunId> JsonViewer::get_rc_run_ids(const MongoViewerConfig &config)
{
    QList<RcRunId> to_return;
//    QList<QFileInfo> files = QDir(get_history_path())
//        .entryInfoList(QDir::NoDotAndDotDot|QDir::Files);
//    for(int i=0; i<files.size(); ++i){
//        QString file_name = files[i].baseName();
//        QString run_index = file_name.split("_").at(0);
//        int run_number = file_name.split("_").at(1).toInt();
//        RcRunId rc(run_index, run_number);
//        to_return.append(rc);
//    }
    return to_return;
}
