﻿#ifndef ABSTRACTVIEWER_H
#define ABSTRACTVIEWER_H

#include <QJsonObject>
#include <QList>
#include <QMap>
#include <QObject>
#include <QStringList>

#include "MongoViewerConfig.h"
#include "base/ConfigSelector.h"
#include "base/RcRunId.h"

class AbstractDB;
class AbstractDbViewer;
class MongoViewer;
class JsonViewer;
class MongoViewerConfig;
class ConfigDocument;

class Viewer : public QObject
{
    Q_OBJECT
public:
    Viewer();
    ~Viewer();
    void update_hosts();
    QList<ConfigDocument> aggregate();
    void update_filters();
    void update_objects();
    void set_object(int i);
    void delete_config();

    QList<MongoHost> hosts;
    QStringList all_program_types;
    QStringList all_program_indexes;
    QStringList all_run_indexes;
    QList<int> all_run_numbers;
    QList<QJsonObject> objects;
    QMap<ConfigSelector, QJsonObject> cs_objects;
    QMap<RcRunId, QJsonObject> ri_objects;
    QJsonObject selected_obj;
    QMap<RcRunId, QDateTime> rc_datetime;

signals:
    void configChanged(const MongoViewerConfig &config);
public slots:
    void setConfig(const MongoViewerConfig &newConfig);

private:
    AbstractDbViewer *getViewer();
    AbstractDB *getDb();

    void clear_all();
    template< typename T >
    QList<T> remove_duplicates(QList<T> &list);
    MongoViewer * const mongo;
    JsonViewer * const json;
    MongoViewerConfig config;
};

#endif // ABSTRACTVIEWER_H
