#include "EvbFileNameDialog.h"
#include "ui_EvbFileNameDialog.h"

EvbFileNameDialog::EvbFileNameDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::EvbFileNameDialog)
{
    ui->setupUi(this);

    for(auto cb : {ui->checkBoxRunIndex, ui->checkBoxRunNumber,
                    ui->checkBoxLocalProgramIndex, ui->checkBoxTimestamp})
        connect(cb, &QCheckBox::toggled,
                this, &EvbFileNameDialog::checkBoxtoggled);

    updateGui();
}

EvbFileNameDialog::~EvbFileNameDialog()
{
    delete ui;
}

void EvbFileNameDialog::setWriterConfig(const EvbWriterConfig &cfg)
{
    if(config == cfg)
        return;
    config = cfg;
    updateGui();
}

void EvbFileNameDialog::on_lineEditPrefix_textEdited(const QString &str)
{
    auto cfg = config;
    cfg.filePrefix = str;
    cfg.fixFilePrefix();

    if(config != cfg) {
        config = cfg;
        emit writerConfigChanged(config);
    }

    if(cfg.filePrefix != ui->lineEditPrefix->text())
        ui->lineEditPrefix->setText(cfg.filePrefix);
}

void EvbFileNameDialog::updateGui() const
{
    ui->lineEditPrefix->setText(config.filePrefix);
    ui->checkBoxRunIndex->setChecked(config.addRunIndex);
    ui->checkBoxRunNumber->setChecked(config.addRunNumber);
    ui->checkBoxLocalProgramIndex->setChecked(config.addLocalProgramIndex);
    ui->checkBoxTimestamp->setChecked(config.addTimestamp);
}

void EvbFileNameDialog::checkBoxtoggled(bool checked)
{
    bool EvbWriterConfig::*ptr = nullptr;
    if(sender() == ui->checkBoxRunIndex)
        ptr = &EvbWriterConfig::addRunIndex;
    if(sender() == ui->checkBoxRunNumber)
        ptr = &EvbWriterConfig::addRunNumber;
    if(sender() == ui->checkBoxLocalProgramIndex)
        ptr = &EvbWriterConfig::addLocalProgramIndex;
    if(sender() == ui->checkBoxTimestamp)
        ptr = &EvbWriterConfig::addTimestamp;
    if(!ptr)
        return;
    if(config.*ptr == checked)
        return;
    config.*ptr = checked;
    emit writerConfigChanged(config);
}
