#ifndef EVBSIGNALROUTER_H
#define EVBSIGNALROUTER_H

class AdjustBufferModel;
class Client;
class ClientManagerFsm;
class Combiner;
class CombinerFsm;
class ConfigurationManager;
class EvBConfigDb;
class EvBConfigDispatcher;
class EvbMainWindow;
class EventBuilderFsm;
class FileWriter;
class RemoteControlServer;
class RunControl;
class StatisticDialog;
class TcpClient;
class TelnetRemoteControl;

class EvbSignalRouter
{
    EvbSignalRouter() = default;
public:
    // Singleton
    static EvbSignalRouter & instance()
    {
        // thread-safe since C++11
        static EvbSignalRouter obj;
        return obj;
    }

    // delete copy and move constructors and assign operators
    EvbSignalRouter(EvbSignalRouter const &) = delete;
    EvbSignalRouter(EvbSignalRouter &&) = delete;
    EvbSignalRouter &operator=(EvbSignalRouter const &) = delete;
    EvbSignalRouter &operator=(EvbSignalRouter &&) = delete;

    void setConfigurationManager(ConfigurationManager *p)
    { configurationManager = p; }

    void setMainWindow(EvbMainWindow *p)
    { mainWindow = p; }

    void setAdjustBufferModel(AdjustBufferModel *p)
    { adjustBufferModel = p; }

    void setEvbFsm(EventBuilderFsm *p)
    { evbFsm = p; }

    void setClientManagerFsm(ClientManagerFsm *p)
    { client_manager_fsm = p; }

    void setCombinerFsm(CombinerFsm *p)
    { combinerFsm = p; }

    void setFileWriter(FileWriter *p)
    { fileWriter = p; }

    void setRemoteControlServer(RemoteControlServer *p)
    { remoteControlServer = p; }

    void setCombiner(Combiner *p)
    { combiner = p; }

    void setEvBConfigDb(EvBConfigDb * p)
    { configDb = p; }

    void setEvBConfigDispatcher(EvBConfigDispatcher *p)
    { configDispatcher = p;}

    void setStatisticDialog(StatisticDialog *p)
    { statisticDialog = p;}

    void setTelnetRemoteControl(TelnetRemoteControl *p)
    { telnetRc = p; }

    void setWidgetRunControl(RunControl * p)
    { widgetRunControl = p; }

    void connectAll();
    void connect_TcpClient(const TcpClient * tcp);

private:
    void connect_PnpServer();
    void connect_RemoteControlServer();
    void connect_Combiner();
    void connect_Fsm();
    void connect_WidgetRunControl();
    void connect_TelnetRemoteControl() const;
    void connect_StatisticDialog();

    ConfigurationManager *configurationManager = nullptr;
    EvbMainWindow *mainWindow = nullptr;
    AdjustBufferModel *adjustBufferModel = nullptr;
    RemoteControlServer *remoteControlServer = nullptr;
    Combiner * combiner = nullptr;
    EvBConfigDb * configDb = nullptr;
    EvBConfigDispatcher *configDispatcher = nullptr;
    RunControl *widgetRunControl = nullptr;
    EventBuilderFsm * evbFsm = nullptr;
    ClientManagerFsm * client_manager_fsm = nullptr;
    CombinerFsm * combinerFsm = nullptr;
    FileWriter * fileWriter = nullptr;
    StatisticDialog * statisticDialog = nullptr;
    TelnetRemoteControl *telnetRc = nullptr;
};

#endif // EVBSIGNALROUTER_H
