#ifndef MPDTLUMAINWINDOW_H
#define MPDTLUMAINWINDOW_H

#include "lib-common/ReadoutAppMainWindow.h"

namespace Ui {
class MpdTluMainWindow;
}

class ConfigKey;
struct DeviceIndexEnabledMap;
class Tlu40LvdsModel;
class Tlu16SfpModel;
class TluTtlIoWidget;
class TluTtlIoDialog;

class MpdTluMainWindow : public ReadoutAppMainWindow
{
    Q_OBJECT

public:
    explicit MpdTluMainWindow(QWidget *parent = nullptr);
    ~MpdTluMainWindow() override;
    DeviceStatusTableViewFancy* getDeviceStatusTableView() override;
    Tlu16SfpModel *getTlu16SfpModel() const;
    Tlu40LvdsModel *getTlu40LvdsModel() const;
    TluTtlIoWidget *getTluTtlIoWidget40Lvds() const;
    TluTtlIoWidget *getTluTtlIoWidget16Sfp() const;

signals:
    void prepareForStop(quint32 stopEvNum);
    void mstream_reset_all(bool doStatClear);
    void performLocalStart() const;
    void performLocalStop() const;
    void printFsmInputs() const;
    void resetCntLvds() const;
    void resetCntSfp() const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig) override;

protected:
    void saveAppGuiSettings(QSettings &) const override;
    void loadAppGuiSettings(const QSettings &) override;
    void updateControlState() override;

private slots:
    void on_pushButtonStart_clicked();
    void on_pushButtonStop_clicked();

    void on_actionShow_Start_Stop_button_toggled(bool showPb);

private:
    void init_connections();
    bool isControlBlocked() const;

private:
    Ui::MpdTluMainWindow *ui;
    TluTtlIoDialog * const tluTtlIoDialog;
};

#endif // MPDTLUMAINWINDOW_H
