#include "MscStreamCfgForm.h"
#include "ui_MscStreamCfgForm.h"

#include "base/gui_utils.h"
#include <optional>

namespace {
constexpr static const char *k_ce_id = "ceCB ";
std::optional<int> getCeCheckBoxId(const QObject *obj)
{
    auto n = obj->objectName();
    if(n.startsWith(k_ce_id)) {
        bool ok;
        auto id = n.remove(0,5).toInt(&ok);
        if(ok)
            return id;
    }
    return {};
}
}

MscStreamCfgForm::MscStreamCfgForm(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::MscStreamCfgForm)
{
    ui->setupUi(this);
}

MscStreamCfgForm::~MscStreamCfgForm()
{
    delete ui;
}

void MscStreamCfgForm::setConfig(const ConfigKey &configKey, const BaseDeviceAppConfig &newConfig)
{
    config = newConfig;
    RecursiveWidgetSignalBlocker msb(this);
    updateGui();
}

void MscStreamCfgForm::setCurDevice(const DeviceIndex &i)
{
    curDevice = i;
    RecursiveWidgetSignalBlocker msb(this);
    updateGui();
}

MscStreamConfig &MscStreamCfgForm::currentConfig()
{
    return config.findDevice(curDevice).mscCore.mstream;
}

const MscStreamConfig &MscStreamCfgForm::currentConfig() const
{
    return config.findDevice(curDevice).mscCore.mstream;
}

void MscStreamCfgForm::on_checkBoxStreamEn_toggled(bool checked)
{
    if(curDevice.isNull()) return;
    auto &dc = currentConfig();
    dc.en = checked;
    emit configChanged(config);
}

void MscStreamCfgForm::on_checkBoxStreamMask_toggled(bool checked)
{
    if(curDevice.isNull()) return;
    auto &dc = currentConfig();
    dc.useMask = checked;
    emit configChanged(config);
}

void MscStreamCfgForm::on_spinBoxStreamStep_valueChanged(int arg1)
{
    if(curDevice.isNull()) return;
    auto &dc = currentConfig();
    dc.slice = static_cast<quint32>(arg1);
    emit configChanged(config);
}

void MscStreamCfgForm::ceMonitorToggled(bool checked)
{
    if(curDevice.isNull()) return;
    auto &dc = currentConfig();
    auto id = getCeCheckBoxId(sender());
    Q_ASSERT(id.has_value());
    if(!id.has_value()) return;
    Q_ASSERT(dc.ceMonitor.size()>id.value());
    if(dc.ceMonitor.size()>id.value())
        dc.ceMonitor[id.value()] = checked;
    emit configChanged(config);
}

void MscStreamCfgForm::updateGui() const
{
    if(curDevice.isNull())
        return;

    const auto &dc = currentConfig();
    ui->spinBoxStreamStep->setValue(dc.slice);
    ui->checkBoxStreamEn->setChecked(dc.en);
    ui->checkBoxStreamMask->setChecked(dc.useMask);
    updateGroupBoxCeMonitor(dc);
}

void MscStreamCfgForm::updateGroupBoxCeMonitor(const MscStreamConfig &dc) const
{
    const int col_cnt = 2;
    ui->groupBoxCeMonitor->setVisible(!dc.ceMonitor.isEmpty());
    const auto &ceMonitor = dc.ceMonitor;
    auto l = ui->gridLayoutCeMonitor;
    std::optional<int> prevMaxId;
    // remove extra, update state for old
    for(int i=0; i<l->count(); ++i) {
        auto item = l->itemAt(i);
        if(!item) continue;
        auto w = item->widget();
        if(!w) continue;
        auto idOpt = getCeCheckBoxId(w);
        Q_ASSERT(idOpt.has_value());
        if(!idOpt.has_value())
            continue;
        auto id = idOpt.value();
        if(id>=ceMonitor.size()) {
            l->removeItem(item);
            delete w;
            --i;
        } else {
            prevMaxId = std::max(prevMaxId.value_or(-1), id);
            auto cb = qobject_cast<QCheckBox*>(w);
            if(cb) {
                cb->setChecked(ceMonitor.value(id));
            }
        }
    }

    for(int i=prevMaxId.value_or(-1)+1; i<ceMonitor.size(); ++i) {
        auto cb = new QCheckBox(ui->groupBoxCeMonitor);
        cb->setChecked(ceMonitor.value(i));
        cb->setObjectName(k_ce_id+QString::number(i));
        cb->setText(QString::number(i));
        connect(cb, &QCheckBox::toggled, this, &MscStreamCfgForm::ceMonitorToggled);
        l->addWidget(cb,i/col_cnt, i%col_cnt);
    }
}
