/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TRIGSTATUS_H
#define TRIGSTATUS_H

#include <QtGlobal>
#include <QString>
#include <QMap>

#include "WrEpState.h"
#include "base/DeviceIndex.h"
#include "device-modules/BaseDeviceStatus.h"
#include "device-modules/RcCounters/RcCounters.h"

struct TrigWrStatus {
    struct WrErrors {
        quint32 OtherErrorCount = 0;
        quint32 CodeErrorCount = 0;
        quint32 LinkLostCount = 0;
        quint32 SyncLostCount = 0;
    };
    void clear() { *this = TrigWrStatus();}
    WrStatusBits Status;
    WrTaiCode curTai;
    WrTaiCode ncuTai;
    WrTaiCode timestamps[4];
    quint32 Uptime = 0;
    WrErrors errors;
};

struct TrigStatCounters
{
    RcCounters rc_counters;
    QMap<int, quint64> xOffHiTimeoutCnt; // <ch, cntVal>
    QMap<int, quint64> xOffLowTimeoutCnt; // <ch, cntVal>
    QMap<int, quint64> xOffEventCnt; // <ch, cntVal>
};

struct TrigCoreStatus
{
    // trig
    quint16 curTrigState = 0;
    quint16 curXOffState = 0;
    quint16 curXOffTimeoutState = 0;
    quint32 spillPlotAddr = 0;
    quint64 globEvNum = 0; // since f/w 28815
    quint16 trigSrc = 0;
};

struct TrigStatus: public BaseDeviceStatus
{
    explicit TrigStatus(const DeviceIndex &i = {});
    TrigStatus (const TrigStatus &) = default;
    TrigStatus &operator=(const TrigStatus &) = default;

    quint64 eventNumber() const;

    TrigCoreStatus coreStatus;

    TrigWrStatus wr;

    TrigStatCounters statCounters;

    // other
    int nch = 0;

};
Q_DECLARE_METATYPE(TrigStatus)


#endif // TRIGSTATUS_H
