#include "CalibrationTriggerWidget.h"
#include "ui_CalibrationTriggerWidget.h"

#include "device-modules/TrcCore/PhysicalTrigger/PhysicalTriggerConfig.h"

class CalibrationTriggerWidgetPrivate
{
public:
    CalibrationTriggerConfig cfg;
};

CalibrationTriggerWidget::CalibrationTriggerWidget(QWidget *parent)
    : QGroupBox(parent)
    , ui(new Ui::CalibrationTriggerWidget)
    , d_ptr(new CalibrationTriggerWidgetPrivate)
{
    ui->setupUi(this);
}

CalibrationTriggerWidget::~CalibrationTriggerWidget()
{
    delete ui;
}

void CalibrationTriggerWidget::setConfig(const CalibrationTriggerConfig &c)
{
    Q_D(CalibrationTriggerWidget);
    d->cfg = c;
    restoreGui();
}

void CalibrationTriggerWidget::on_checkBoxEn_toggled(bool en)
{
    Q_D(CalibrationTriggerWidget);
    d->cfg.en = en;
    emit cfgUpdated(d->cfg);
}

void CalibrationTriggerWidget::on_spinBoxPeriod_valueChanged(int p)
{
    Q_D(CalibrationTriggerWidget);
    d->cfg.period = static_cast<quint32>(p);
    emit cfgUpdated(d->cfg);
}

void CalibrationTriggerWidget::on_spinBoxCount_valueChanged(int c)
{
    Q_D(CalibrationTriggerWidget);
    d->cfg.count = static_cast<quint16>(c);
    emit cfgUpdated(d->cfg);
}

void CalibrationTriggerWidget::on_spinBoxOffset_valueChanged(int o)
{
    Q_D(CalibrationTriggerWidget);
    d->cfg.offset = static_cast<quint32>(o);
    emit cfgUpdated(d->cfg);
}

void CalibrationTriggerWidget::restoreGui() const
{
    Q_D(const CalibrationTriggerWidget);
    ui->checkBoxEn->setChecked(d->cfg.en);
    ui->spinBoxCount->setValue(d->cfg.count);
    ui->spinBoxPeriod->setValue(static_cast<int>(d->cfg.period));
    ui->spinBoxOffset->setValue(static_cast<int>(d->cfg.offset));
}
