/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ConfigSelector.h"

#include "base/RcRunId.h"

QString ConfigSelector::toWindowTitle() const
{
    QStringList l{daqModule.prettyName()};
    if(!configIndexName.program_index.isDefault())
        l << "#"+configIndexName.program_index.toString();
    if(!configIndexName.configuration_name.isDefault())
        l << "(" + configIndexName.configuration_name + ")";
    return l.join(" ");
}

bool operator<(const ConfigSelector &l, const ConfigSelector &r)
{
    if (l.daqModule == r.daqModule)
    return l.configIndexName < r.configIndexName;
    return l.daqModule < r.daqModule;
}

QString ConfigSelector::toString() const
{
    return QString("Program type = %1; program index = %2; configuration name = %3")
            .arg(getType().toString())
            .arg(getIndex().toString())
            .arg(getConfigurationName());
}

QDebug operator<<(QDebug dbg, const ConfigSelector &r)
{
    dbg.nospace().noquote()
            << "ConfigSelector(daqModule: " << r.daqModule
            // << ", configIndexName: " << r.configIndexName
            << ", program_index: " << r.configIndexName.program_index
            << ", configuration_name: " << r.configIndexName.configuration_name
            << ")";
    return dbg;
}
