/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "HexDump.h"

class HexDumpImpl
{
public:
    HexDumpImpl() = default;
    HexDumpImpl(int lineBytes, int groupBy)
        : XD_LINE_BYTES(lineBytes), XD_GROUPBY(groupBy) {}

    QStringList hexdump(const void *ptr, int len) const;

protected:
    QString hexdump_line(int address, const void *ptr, int len) const;
    static char printable(char c, char fill);
private:
    int XD_LINE_BYTES = 32;
    int XD_GROUPBY = 4;
};

QStringList HexDump::toStringList(const void *ptr, int len)
{
    HexDumpImpl impl;
    return impl.hexdump(ptr, len);
}

char HexDumpImpl::printable(char c, char fill)
{
    if ((c < 0x20) || (c > 0x7e))
        return fill;
    return c;
}

QString HexDumpImpl::hexdump_line(int address, const void *ptr, int len) const
{
    if (!ptr || len <= 0 || len > XD_LINE_BYTES)
        return {};
    QString buf;
    QString asciibuf;
    for (int i=0; i<len; i++) {
        const char c = ((const char *)ptr)[i];
        asciibuf += printable(c, '.');
        auto pp = ((const unsigned char *)ptr)[i];
        buf += QString("%1").arg(pp, 2, 16, QChar('0'));
        if (i % XD_GROUPBY == XD_GROUPBY - 1) {
            buf += " ";
        }
    }
    for (int i=len; i<XD_LINE_BYTES; i++) {
        buf += "  ";
        if (i % XD_GROUPBY == XD_GROUPBY - 1) {
            buf += " ";
        }
    }
    return QString("  %1: %2  %3").arg(address, 4, 16, QChar('0')).arg(buf, asciibuf);
}

QStringList HexDumpImpl::hexdump(const void *ptr, int len) const
{
    if (len <= 0) {
        return {"hexdump: empty"};
    }
    QStringList sl;
    int i = 0;
    while (i <= len) {
        int start = i / XD_LINE_BYTES * XD_LINE_BYTES;
        int count = len - start;
        if (count > XD_LINE_BYTES)
            count = XD_LINE_BYTES;
        sl << hexdump_line(start, (const char *)(ptr) + i, count);
        i += XD_LINE_BYTES;
    }
    return sl;
}
