/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RcRunId.h"

#include <QDataStream>
#include <QDebug>

bool RcRunId::isValid() const
{
    return !runIndex.isEmpty() && !uuid.isNull();
}

QString RcRunId::toString() const
{
    return QString("%1 %2").arg(runIndex).arg(runNumber);
}

QDebug operator<<(QDebug dbg, const RcRunId &r)
{
    dbg.noquote() << QString("RunId(index: \"%1\", number: %2, uuid: %3)")
        .arg(r.runIndex).arg(r.runNumber).arg(r.uuid.toString());
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const RcRunId &value)
{
    out << value.runIndex << value.runNumber << value.uuid;
    return out;
}

QDataStream &operator>>(QDataStream &in, RcRunId &value)
{
    RcRunId tmp;
    in >> tmp.runIndex >> tmp.runNumber >> tmp.uuid;
    if (in.status() == QDataStream::Ok) {
        value = tmp;
    }
    return in;
}
