#include "RunDocumentConverter.h"

#include "RunDocument.h"

namespace {
// RunDocumentMeta
constexpr const char *k_topProgram = "topProgram";
constexpr const char *k_startDateTime = "startDateTime";

// RunDocument
constexpr const char *k_runId = "runId";
constexpr const char *k_programs = "programs";
constexpr const char *k_devHw = "devHw";
constexpr const char *k_devFw = "devFw";
constexpr const char *k_devCal = "devCal";
constexpr const char *k_meta = "meta";
}

template<>
bool AbstractConfigConverter<RunDocumentMeta>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RunDocumentMeta>::toJsonObject(const RunDocumentMeta &in)
{
    QVariantMap m;
    m[k_topProgram] = QVariant::fromValue(in.topProgram);
    m[k_startDateTime] = in.startDateTime.toString(Qt::ISODate);
    return fromVariantMap(m);
}

template<>
RunDocumentMeta AbstractConfigConverter<RunDocumentMeta>::fromJsonObject(const QJsonObject &in)
{
    RunDocumentMeta out;
    update_value(in, k_topProgram, &out.topProgram);
    out.startDateTime = QDateTime::fromString(in[k_startDateTime].toString(),
                                              Qt::ISODate);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<RunDocumentMeta>::toJsonValue(const RunDocumentMeta &in)
{
    return toJsonObject(in);
}

template<>
RunDocumentMeta AbstractConfigConverter<RunDocumentMeta>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RunDocumentMeta>::registerSubTypes()
{
    AbstractConfigConverter<ClientIndex>::registerTypes();
}


template<>
bool AbstractConfigConverter<RunDocument>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RunDocument>::toJsonObject(const RunDocument &in)
{
    QVariantMap m;
    if(in.runId != RcRunId())
        m[k_runId] = QVariant::fromValue(in.runId);
    if(in.meta != RunDocumentMeta())
        m[k_meta] = QVariant::fromValue(in.meta);
    if(!in.programs.isEmpty())
        m[k_programs] = ConfigConverterUtil::toJsonValue(in.programs);
    if(!in.devHw.isEmpty())
        m[k_devHw] = ConfigConverterUtil::toJsonValue(in.devHw);
    if(!in.devFw.isEmpty())
        m[k_devFw] = ConfigConverterUtil::toJsonValue(in.devFw);
    if(!in.devCal.isEmpty())
        m[k_devCal] = ConfigConverterUtil::toJsonValue(in.devCal);
    return fromVariantMap(m);
}

template<>
RunDocument AbstractConfigConverter<RunDocument>::fromJsonObject(const QJsonObject &in)
{
    RunDocument out;
    update_value(in, k_runId, &out.runId);
    update_value(in, k_meta, &out.meta);
    out.programs = ConfigConverterUtil::toMap<ClientIndex, QJsonObject>(in[k_programs]);
    out.devHw  = ConfigConverterUtil::toMap<DeviceIndex, QJsonObject>(in[k_devHw]);
    out.devFw  = ConfigConverterUtil::toMap<DeviceIndex, QJsonObject>(in[k_devFw]);
    out.devCal = ConfigConverterUtil::toMap<DeviceIndex, QJsonObject>(in[k_devCal]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<RunDocument>::toJsonValue(const RunDocument &in)
{
    return toJsonObject(in);
}

template<>
RunDocument AbstractConfigConverter<RunDocument>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RunDocument>::registerSubTypes()
{
    AbstractConfigConverter<ClientIndex>::registerTypes();
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<RcRunId>::registerTypes();
    AbstractConfigConverter<RunDocumentMeta>::registerTypes();
}
