/*
**    Copyright 2013-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DECODERCONFIG_H
#define DECODERCONFIG_H

#include <ostream>
#include <QMap>
#include <QMetaType>

#include "CalAvgConfig.h"
#include "FitterGausProfile.h"

typedef int prof_id_t;

class DecoderChannelConfig
{
public:
    friend bool operator==(const DecoderChannelConfig &, const DecoderChannelConfig &) = default;
    bool isValid() const;

    std::vector<double> fir; //!< FIR digital filters
    //  double  pwmin;       //!< fitted pulse width - minimum
    //  double  pwmax;       //!< fitted pulse width - maximum
};

Q_DECLARE_METATYPE(DecoderChannelConfig)


class DecoderConfig
{
public:
    bool isValid() const;
    friend bool operator==(const DecoderConfig &, const DecoderConfig &) = default;

    DecoderChannelConfig channels;
    CalAvgConfig calavg;
    dsplib::FitterGausProfile fitprof;
};

Q_DECLARE_METATYPE(DecoderConfig)

#endif // DECODERCONFIG_H
