#ifndef DEVICEMGRFSM_H
#define DEVICEMGRFSM_H

#include "BaseFsm.h"
#include <QMap>

#include "base/ConfigKey.h"
#include "base/DeviceIndex.h"

class DeviceIndexEnabledMap;

struct DeviceMgrFsmInput : public BaseFsmInput
{
    ConfigKey setupKey;
    QElapsedTimer lastSetupKeyUpdate;
    DeviceIndexSet devSet;
    QMap<DeviceIndex, ConfigKey> deviceSetupKeys;
};

class DeviceMgrFsm : public BaseFsm
{
    Q_OBJECT
public:
    explicit DeviceMgrFsm(QObject *parent = nullptr);
    virtual bool isTransitionAllowed(const FsmState &toState) const override;

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &newList);
    void setMainSetupKey(const ConfigKey &key);
    void setDevSetupKey(const DeviceIndex &index, const ConfigKey &key);

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &);

protected slots:
    virtual bool evaluate() override;

protected:
    virtual void printInputString(QDebug dbg) const override;
    virtual qint64 stateTimeoutMs(const FsmState &state) const override;
    inline DeviceMgrFsmInput *getInput() {
        return static_cast<DeviceMgrFsmInput*>(BaseFsm::getInput());
    }
    inline const DeviceMgrFsmInput *getInput() const {
        return static_cast<const DeviceMgrFsmInput*>(BaseFsm::getInput());
    }

    bool checkDevKey(const ConfigKey &mainKey, const QMap<DeviceIndex, ConfigKey> &keyMap) const;
    bool allDevicesWritten() const;
    bool setupKeyGuargExpired() const;

    const int DEVICE_CONFIG_WRITE_TIMEOUT = 1000; // in ms
};

QDebug operator<<(QDebug dbg, const DeviceMgrFsmInput &in);


#endif // DEVICEMGRFSM_H
