/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICETABLE_H
#define DEVICETABLE_H

#include "base/DeviceIndexEnabled.h"
#include "DeviceTableEntry.h"

class DeviceTable
{
public:
    typedef typename QVector<DeviceTableEntry>::iterator iterator;
    typedef typename QVector<DeviceTableEntry>::const_iterator const_iterator;

    // basic ops
    int size() const { return data.size(); }
    bool contains(const DeviceIndex &index) const {
        return find(index) != end();
    }
    DeviceIndexSet keys() const;
    void insertStatic(const DeviceTableEntry &r); // insert or merge
    void insertDynamic(const DeviceTableEntry &r); // insert or merge
    void removeStatic(const DeviceIndexSet &r);

    // transformations
//    DeviceTable filterDynamic(bool dynamic = true) const;
    DeviceTable filterStaticEnabled(bool enabled = true) const;

    // DynamicFlag == Static
//    DeviceIndexEnabledVector staticToVector() const;
    static DeviceIndexEnabledMap staticToMap(const DeviceTable &deviceTable);
//    static DeviceIndexEnabledVector staticToVector(const DeviceTable &deviceTable);
    static DeviceTable staticFromMap(const DeviceIndexEnabledMap &v);
    static DeviceTable staticFromVector(const DeviceIndexEnabledVector &v);

    friend bool operator==(const DeviceTable &, const DeviceTable &) = default;

    DeviceTableEntry value(const DeviceIndex &index) const;
    DeviceTableEntry value(int row) const;
    int rowOf(const DeviceIndex &deviceIndex) const;

    iterator begin() { return data.begin(); }
    iterator end() { return data.end(); }
    const_iterator begin() const { return data.begin(); }
    const_iterator end() const { return data.end(); }

    iterator find(int row);
    iterator find(const DeviceIndex &deviceIndex);
    const_iterator find(int row) const;
    const_iterator find(const DeviceIndex &deviceIndex) const;

private:
    DeviceIndexSet getIndices(DeviceTableEntry::DynamicFlag dynamic) const;
    void remove(const DeviceIndexSet &r);

    QVector<DeviceTableEntry> data;
};

#endif // DEVICETABLE_H
