/*
**    Copyright 2014-2020 Ivan Filippov
**    Copyright 2014-2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DISCOVERDIALOGPRIVATE_H
#define DISCOVERDIALOGPRIVATE_H

#include "DeviceTableModel.h"
#include "ProxyChain.h"
#include "base/DeviceId.h"
#include "base/DeviceIndexEnabled.h"

namespace Ui {
class DiscoverDialog;
}

class DiscoverDialog;
class QAbstractButton;

class DiscoverDialogPrivate
{
public:
    DiscoverDialogPrivate(DiscoverDialog *q);
    ~DiscoverDialogPrivate();
    void setDeviceList(const DeviceIndexEnabledMap &);

private:
    void apply();

    void on_pushButtonManualAdd_clicked();
    void on_pushButtonAdd_clicked();
    void on_pushButtonRemove_clicked();
    void on_checkBoxShowMac_toggled(bool checked);
    void on_listWidgetTypes_itemSelectionChanged();

    void on_tableViewAvailable_doubleClicked(const QModelIndex &index);
    void on_tableViewSelected_doubleClicked(const QModelIndex &index);
    void on_buttonBox_clicked(QAbstractButton *button);


    DeviceIndex deviceIndexFromAvail(const QModelIndex &index) const;
    DeviceIndex deviceIndexFromSelected(const QModelIndex &index) const;
    void moveToStatic(const QModelIndex &index) const;
    void moveToDynamic(const QModelIndex &index) const;

    Q_DISABLE_COPY(DiscoverDialogPrivate)
    Q_DECLARE_PUBLIC(DiscoverDialog)
    DiscoverDialog * const q_ptr;

    Ui::DiscoverDialog * const ui;
    ProxyChain * const modelFactory;
    DeviceTableModel * const model;
    QSet<DeviceId> deviceIdFilter;
};

#endif // DISCOVERDIALOGPRIVATE_H
