/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef LLDPDISCOVERYTABLEMODEL_H
#define LLDPDISCOVERYTABLEMODEL_H

#include "mvc/DeviceIndexTableModel.h"
#include "mldiscover/LldpDiscovery.h"
#include <optional>

class QTimer;

class LldpDiscoveryTableModel : public DeviceIndexTableModel
{
    Q_OBJECT
public:
    enum Columns {
        COL_ONLINE,
        COL_DEVICE_INDEX,
        COL_DEVICE_ID,
        COL_SERIAL,
        COL_HW_VER,
        COL_FIRM_WARE,
        COL_SLOT,
        COL_IP,
        COL_MAC,
        COL_MASTER,
        COL_M_STREAM,
        COL_TOTAL
    };
    explicit LldpDiscoveryTableModel(QObject *parent = nullptr);

    int rowCount(const QModelIndex &parent = {}) const override;
    int columnCount(const QModelIndex &parent = {}) const override;

    DeviceIndex deviceIndexOf(int row) const override;
    int rowOf(const DeviceIndex &deviceIndex) const override;

    QVariant data(const QModelIndex &index, int role) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const override;

    QMap<int, bool> defaultColumnState() const override;

public slots:
    void lldpCacheUpdated(const LldpInfoCache &input);
    void pollData();

protected:
    void insertRow(const DeviceIndex &deviceIndex, const LldpInfo &data);
    void removeRow(const DeviceIndex &deviceIndex);

private:
    QVariant getDisplayData(int column, const DeviceIndex &deviceIndex) const;
    QVariant getToolTipData(int column, const DeviceIndex &deviceIndex) const;
    QVariant getUserData(int column, const DeviceIndex &deviceIndex) const;
    QString makeIpPortStr(bool locked, quint32 ip, quint16 port) const;
    void checkPendingData();
    void checkForExpire();

    LldpInfoCache lldpData;
    std::optional<LldpInfoCache> pendingData;
    QMap<DeviceIndex, bool> expiredState;
    QTimer * const pollDataTimer;
};

#endif // LLDPDISCOVERYTABLEMODEL_H
