/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ASYNCDEVICECOMMANDER_H
#define ASYNCDEVICECOMMANDER_H

#include "DeviceCommand.h"
#include "base/DeviceIndexEnabled.h"

#include <QMap>
#include <QObject>

class AsyncDeviceCommander : public QObject
{
    Q_OBJECT
public:

    struct MultiDeviceTask {
        bool isComplete() const {
            const auto &keys = commands.keys();
            for (const auto &k : keys)
                if (!results.contains(k))
                    return false;
            return true;
        }
        DeviceCommand command;
        QMap<DeviceIndex, DeviceCommand> commands;
        QMap<DeviceIndex, DeviceCommand> results;
    };

    explicit AsyncDeviceCommander(QObject *parent = nullptr);

signals:
    void commandComplete(const DeviceCommand &command);
    void submitDeviceCommand(const DeviceIndex &index, const DeviceCommand &command);

public slots:
    void submitCommand(const DeviceCommand &newCommand);
    void setDeviceList(const DeviceIndexEnabledMap &new_devices)
    {
        deviceList = new_devices;
    }
    void deviceCommandComplete(const DeviceIndex &index, const DeviceCommand &completeCommand);

private:
    void checkTaskResults(const MultiDeviceTask &task);
    void completeNoDevices(const DeviceCommand &newCommand);

    DeviceIndexEnabledMap deviceList;
    QMap<int, MultiDeviceTask> tasks;
};

#endif // ASYNCDEVICECOMMANDER_H
