/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DlnpCalibrationTriggerConfigConverter.h"
#include "DlnpCalibrationTrigger/DlnpCalibrationTriggerConfig.h"


constexpr const char *k_en = "en";
constexpr const char *k_period = "period";
constexpr const char *k_count = "count";
constexpr const char *k_offset = "offset";

// =========================

template<>
bool AbstractConfigConverter<DlnpCalibrationTriggerConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<DlnpCalibrationTriggerConfig>::toJsonObject(const DlnpCalibrationTriggerConfig &in)
{
    QVariantMap m;
    m[k_en] = QVariant::fromValue(in.en);
    m[k_period] = QVariant::fromValue(in.period);
    m[k_count] = QVariant::fromValue(in.count);
    m[k_offset] = QVariant::fromValue(in.offset);
    return fromVariantMap(m);
}

template<>
DlnpCalibrationTriggerConfig AbstractConfigConverter<DlnpCalibrationTriggerConfig>::fromJsonObject(const QJsonObject &in)
{
    DlnpCalibrationTriggerConfig out;
    update_value(in, k_en, &out.en);
    update_value(in, k_period, &out.period);
    update_value(in, k_count, &out.count);
    update_value(in, k_offset, &out.offset);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<DlnpCalibrationTriggerConfig>::toJsonValue(const DlnpCalibrationTriggerConfig &in)
{
    return toJsonObject(in);
}

template<>
DlnpCalibrationTriggerConfig AbstractConfigConverter<DlnpCalibrationTriggerConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<DlnpCalibrationTriggerConfig>::registerSubTypes()
{
}
