/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MODULARDEVICECONFIG_H
#define MODULARDEVICECONFIG_H

#include <QMetaType>

#include "Ad5328/Ad5328Config.h"
#include "Ad5622/Ad5622Config.h"
#include "AdcErc/AdcErcConfig.h"
#include "AdcLemoMux/AdcLemoMuxConfig.h"
#include "AdcMpdTrig/AdcMpdTrigConfig.h"
#include "AdcSystem/DominoSettings.h"
#include "BoardGpio/BoardGpioConfig.h"
#include "ClockControl/ClockControlConfig.h"
#include "CrosspointSwitch/CrosspointSwitchConfig.h"
#include "DeserializersControl/AdcSerDesDelay.h"
#include "DlnpCalibrationTrigger/DlnpCalibrationTriggerConfig.h"
#include "FeLinkStatus/FeLinkConfig.h"
#include "FirFilter/FirFilterConfig.h"
#include "FlashProg/FlashProgModuleConfig.h"
#include "HitCore/HitCoreConfig.h"
#include "Hrb/HrbModuleConfig.h"
#include "MscCore/MscCoreConfig.h"
#include "NetworkPort/NetworkPortConfig.h"
#include "RunLogic/RunLogicConfig.h"
#include "SpillControl/SpillConfig.h"
#include "StatReadout/StatReadoutConfig.h"
#include "TdcCore/TdcCoreConfig.h"
#include "TqdcCore/TqdcCoreConfig.h"

#include "Tlu16SfpTrigger/Tlu16SfpTriggerConfig.h"
#include "Tlu40LvdsLuminosity/Tlu40LvdsLuminosityConfig.h"
#include "Tlu40LvdsTofTrigger/Tlu40LvdsTofTriggerConfig.h"
#include "TluDecDes/TluDecDesConfig.h"
#include "TluTtlIoControl/TluTtlIoControlConfig.h"

#include "TrcCore/TrcCoreConfig.h"
#include "TrigCtrlStat/TrigCtrlStatConfig.h"
#include "Ttb/TtbModuleConfig.h"
#include "TtlIo/TtlIoConfig.h"
#include "Ttvxs/TtvxsCoreConfig.h"
#include "WaveBlc/WaveBlcConfig.h"
#include "WaveRec/WaveRecConfig.h"
#include "WaveTrig/WaveTrigConfig.h"
#include "base/BaseDeviceConfig.h"
#include "cal/CalResult.h"

class ModularDeviceConfig : public BaseDeviceConfig
{
public:
    friend bool operator==(const ModularDeviceConfig &, const ModularDeviceConfig &) = default;
    void mergeLocalData(const ModularDeviceConfig &cfg);
    void unify_config(const ModularDeviceConfig &cfg);

    // modules
    QMap<int, Ad5328Config> ad5328Configs;
    Ad5622Config ad5622;
    AdcErcConfig adcErc;
    AdcLemoMuxCtrlConfig adcLemoMux;
    AdcMpdTrigConfig adcMpdTrig;
    AdcSerDesDelayMap serDesDelay;
    BoardGpioConfig boardGpio;
    ClockControlConfig clockControl;
    CrosspointSwitchConfig crosspointSwitch;
    DlnpCalibrationTriggerConfig dlnpCalibrationTrigger;
    FeLinkConfig feLink;
    FlashProgModuleConfig flashProg;
    FirFilterConfig firFilter;
    HitCoreConfig hitCore;
    NetworkPortConfig networkPort;
    RunLogicConfig runLogic;
    SpillConfig spillControl;
    StatReadoutConfig statReadout;

    Tlu16SfpTriggerConfig tlu16SfpTrigger;
    Tlu40LvdsLuminosityConfig tlu40lvdsLuminosity;
    Tlu40LvdsTofTriggerConfig tlu40lvdsTofTrigger;
    TluDecDesConfig tluDecDes;
    TluTtlIoControlConfig tluTtlIo;

    TrigCtrlStatConfig trigCtrlStat;
    TtlIoConfig ttlIo;
    WaveBlcConfig waveBlc;
    WaveRecConfig waveRec;
    WaveTrigConfig waveTrig;

    bool periodicalAdcDelayCheck = false;

    // cores
    CalResult calResult;
    DominoSettings adcSystemCore;
    HrbModuleConfig hrb;
    MscCoreConfig mscCore;
    TdcCoreConfig tdcCore;
    TqdcCoreConfig tqdcCore;
    TrcCoreConfig trcCore;
    TtbModuleConfig ttb;
    TtvxsCoreConfig ttvxsCore;
};
Q_DECLARE_METATYPE(ModularDeviceConfig)

#endif // MODULARDEVICECONFIG_H
