#include "MscStreamModule.h"

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

#include "MscStreamConfig.h"

namespace {
enum {
    //Stream readout CSR
    REG_CTRL = 0x0,
    REG_CE_MONITOR_MASK = 0x1, //External count enable monitor mask
    REG_BUILD_TIMEOUT = 0x3,
    REG32_SLICETIME = 0x6,
};

enum {
    REG_CTRL_BIT_USE_MASK = 1 << 14,
    REG_CTRL_BIT_ENABLE = 1 << 15,
};
}

QList<AFI_SDB_ID> MscStreamModule::getSupportedSdbIds()
{
    return {AFI_SDB_MSC_STREAM_READOUT};
}

RegOpVector MscStreamModule::setConfigRegOp(const MscStreamConfig &cfg)
{
    RegOpVector r;
    r.RegWrite16(REG_CTRL, getControlReg(cfg));
    r.RegWrite16(REG_CE_MONITOR_MASK, getCeMonitorMask(cfg));
    r.RegWrite16(REG_BUILD_TIMEOUT, cfg.timeout);
    r.RegWrite32(REG32_SLICETIME, (cfg.slice/8)*8);
    return r;
}

bool MscStreamModule::writeConfig(const MscStreamConfig &cfg)
{
    auto r = setConfigRegOp(cfg);
    return regOpExecRebased(r);
}

quint16 MscStreamModule::getControlReg(const MscStreamConfig &cfg) const
{
    quint16 v=0;
    if(cfg.en)
        v |= REG_CTRL_BIT_ENABLE;
    if(cfg.useMask)
        v |= REG_CTRL_BIT_USE_MASK;
    return v;
}

quint16 MscStreamModule::getCeMonitorMask(const MscStreamConfig &cfg) const
{
    quint16 ret=0;
    for(int i=0; i<cfg.ceMonitor.size(); ++i)
        if(cfg.ceMonitor[i])
            ret |= 1<<i;
    return ret;
}
