/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "OneWire.h"
#include "crc8_dallas_maxim.h"

static const quint8 OW_ID_DS18B20 = 0x28;

quint64 OneWire::getSerial(quint64 ow_id)
{
    return (ow_id << 8) >> 16;
}

bool OneWire::oneWireCrcValid(quint64 ow_id)
{
    const quint8 ow_crc = ow_id >> 56;
    // const int testID = OW_ID_DS18B20;
    unsigned char crc = 0;
    for(int i=0; i<7; i++)
        crc = crc8_dallas_maxim((ow_id >> i*8) & 0xFF, crc);
    return crc == ow_crc;
}

bool OneWire::oneWireIdValid(quint64 ow_id)
{
    return (ow_id & 0xFF) == OW_ID_DS18B20;
}

double OneWire::convertOneWireTemp(quint16 rawTemp)
{
    rawTemp &= 0xFFF;
    if (rawTemp & 0x800) rawTemp = -(rawTemp&0x7FF);
    return 0.0625 * rawTemp;
}
