#include "StatReadoutModule.h"

#include "base/DeviceId.h"
#include "device-modules/MregBusInterface.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"
#include "StatReadoutConfig.h"

enum {
    // Statistics Readout CSR Registers
    REG_STAT_BASE = 0x900,
    REG_STAT_CTRL       = 0x00,
    REG_STAT_PERIOD     = 0x02,
    REG_STAT_BLOCK_SIZE = 0x04,
    REG_STAT_RAM_ADDR   = 0x05,
    REG_STAT_RAM_DATA   = 0x06
};

StatReadoutModule::StatReadoutModule(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
    if (!isAvailable()) {
        if  (getDeviceId() == DEVICE_ID_TTVXS)
            setBaseAddr(REG_STAT_BASE, 0xFF);
    }
}

QList<AFI_SDB_ID> StatReadoutModule::getSupportedSdbIds()
{
    return {AFI_SDB_STAT_READOUT_CTRL};
}

bool StatReadoutModule::writeConfig(const StatReadoutConfig &config)
{
    RegOpVector r;
    r.RegWrite16(REG_STAT_CTRL, config.enable ? 1 : 0);

    r.RegWrite16(REG_STAT_PERIOD, config.period);

    auto regs = config.preset.regs;
    if (regs.isEmpty()) {
        qCritical() << "Statistic preset is empty; init it with one address equal to 0";
        regs << 0;
    }
    r.RegWrite16(REG_STAT_BLOCK_SIZE, static_cast<quint16>(regs.size() - 1));
    for (quint16 i = 0; i < regs.size(); ++i) {
        r.RegWrite16(REG_STAT_RAM_ADDR, i);
        r.RegWrite16(REG_STAT_RAM_DATA, regs[i]);
    }

    return regOpExecRebased(r);
}
