#ifndef TLUTTLIOCONTROLCONFIG_H
#define TLUTTLIOCONTROLCONFIG_H

#include <QMetaType>
#include <QVector>

enum class TluTtlIoInput
{
    Trigger = 0x00,
    Veto    = 0x01,
    LTR_0   = 0x02,
    LTR_1   = 0x03,
    LTR_2   = 0x04,
    LTR_3   = 0x05,
};

enum class TluTtlIoOutput
{
    AcceptedTrigger         = 0x00,
    RejectedTrigger         = 0x01,
    Busy                    = 0x02,
    TOF_detector_trig       = 0x03,
    Luminosity_detector_trig= 0x04,
    Pps                     = 0x05,
    Logic0                  = 0x06,
    Logic1                  = 0x07,
};

typedef union reg_tlu_ttl_io_t {
    void setInput(TluTtlIoInput i);
    void setOutput(TluTtlIoOutput o);
    TluTtlIoInput getInput() const;
    TluTtlIoOutput getOutput() const;
    QString toString() const;

    uint16_t all = 0;
    struct {
        uint16_t cfg: 15;
        uint16_t isOutput: 1;
    } bit;
} reg_tlu_ttl_io_t;
Q_DECLARE_METATYPE(reg_tlu_ttl_io_t)

inline bool operator==(const reg_tlu_ttl_io_t &l, const reg_tlu_ttl_io_t &r)
{
    return l.all == r.all;
}

struct TluTtlIoControlConfig
{
    static QList<TluTtlIoInput> getInputs();
    static QList<TluTtlIoOutput> getOutputs();
    static QString getName(const TluTtlIoInput &type);
    static QString getName(const TluTtlIoOutput &type);
    static QString getPrettyName(const TluTtlIoInput &type);
    static QString getPrettyName(const TluTtlIoOutput &type);

    friend bool operator==(const TluTtlIoControlConfig &l, const TluTtlIoControlConfig &r) = default;

    QVector<reg_tlu_ttl_io_t> ttlCfg;
};
Q_DECLARE_METATYPE(TluTtlIoControlConfig)

#endif // TLUTTLIOCONTROLCONFIG_H
