#ifndef PHISICALTRIGGERMODULE_H
#define PHISICALTRIGGERMODULE_H

#include <optional>

#include "device-modules/BaseDeviceModule.h"

struct PhysicalTriggerBuildParams;
struct PhysicalTriggerConfig;

struct PhysicalTriggerStatus;

class RcTrigModule;
class SpillControlModule;
class RcCountersModule;

enum DevCountersTypes : int;

template<typename  T> class TrcSubModule;

class PhysicalTriggerModule : public BaseDeviceModule<PhysicalTriggerModule>
{
public:
    explicit PhysicalTriggerModule(const MregBusInterface &bus);
    ~PhysicalTriggerModule() override;

    static QList<AFI_SDB_ID> getSupportedSdbIds();

    void post_init() final;
    void softClear();

    PhysicalTriggerBuildParams getBuildParam();

    bool writeConfig(const PhysicalTriggerConfig &cfg);
    bool readStatus(PhysicalTriggerStatus *status);

private:
    bool setCountersLock(bool lock);
    inline quint16 getControlReg() const;

    TrcSubModule<RcTrigModule> * const rcTrigMod;
    TrcSubModule<SpillControlModule> * const spillMod;
    QMap<DevCountersTypes, TrcSubModule<RcCountersModule> *> const rcCountersMods;

    bool oscilogramDelayed = false;
};

#endif // PHISICALTRIGGERMODULE_H
