/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ClockControlDialog.h"
#include "ClockControlWidget.h"

#include <QGridLayout>
#include <QAbstractButton>
#include <QDialogButtonBox>

ClockControlDialog::ClockControlDialog(QWidget *parent)
    : QDialog(parent)
    , widget(new ClockControlWidget(this))
{
    setWindowTitle("Clock control");
    setModal(false);
    auto layout = new QGridLayout(this);
    layout->addWidget(widget);

    connect(this, &ClockControlDialog::setClockControlBp,
            widget, &ClockControlWidget::setClockControlBp);
    connect(this, &ClockControlDialog::setDeviceList,
            widget, &ClockControlWidget::setDeviceList);
    connect(this, &ClockControlDialog::setClockControlStatus,
            widget, &ClockControlWidget::setClockControlStatus);
    connect(this, &ClockControlDialog::setClockControlConfig,
            widget, &ClockControlWidget::setConfig);
    connect(widget, &ClockControlWidget::configChanged,
            this, &ClockControlDialog::clockControlConfigChanged);

    auto buttonBox = new QDialogButtonBox(this);
    buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::RestoreDefaults |
                                  QDialogButtonBox::Close);
    layout->addWidget(buttonBox);
    connect(buttonBox, &QDialogButtonBox::accepted,
            this, &QDialog::accept);
    connect(buttonBox, &QDialogButtonBox::rejected,
            this, &QDialog::reject);
    for (auto *button : buttonBox->buttons()) {
        switch (buttonBox->standardButton(button)) {
        case QDialogButtonBox::RestoreDefaults:
            connect(button, &QAbstractButton::clicked,
                    this, &ClockControlDialog::restoreDefaults);
            break;
        default:;
        }
    }
}

void ClockControlDialog::restoreDefaults()
{
    emit setClockControlConfig({});
}

