#include "CommonSpinBoxDelegate.h"

#include <QDebug>
#include <QSpinBox>
#include <optional>

CommonSpinBoxDelegate::CommonSpinBoxDelegate(QObject *parent)
    : QItemDelegate(parent)
{
}

QWidget *CommonSpinBoxDelegate::createEditor(QWidget *parent,
                                                 const QStyleOptionViewItem &/* option */,
                                                 const QModelIndex &index) const
{
    const auto &var = index.model()->data(index, Qt::EditRole);
    if (var.isNull())
        return {};

    auto spinBox = new QSpinBox(parent);
    connect(spinBox, QOverload<int>::of(&QSpinBox::valueChanged),
            this, &CommonSpinBoxDelegate::dataChanged);

    updateSbByModel(spinBox, index);

    return spinBox;
}

void CommonSpinBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    auto spinBox = static_cast<QSpinBox*>(editor);
    if (!spinBox)
        return;
    updateSbByModel(spinBox, index);
}

void CommonSpinBoxDelegate::setModelData(QWidget *editor,
                                             QAbstractItemModel *model,
                                             const QModelIndex &index) const
{
    auto spin_box = static_cast<QSpinBox*>(editor);
    spin_box->interpretText();
    int value = spin_box->value();
    if(model->data(index, Qt::EditRole).toInt() != value)
        model->setData(index, value, Qt::EditRole);
}

void CommonSpinBoxDelegate::updateEditorGeometry(QWidget *editor,
                                                     const QStyleOptionViewItem &option,
                                                     const QModelIndex &/* index */) const
{
    editor->setGeometry(option.rect);
}

void CommonSpinBoxDelegate::dataChanged()
{
    commitData(qobject_cast<QWidget*>(sender()));
}

void CommonSpinBoxDelegate::updateSbByModel(QSpinBox *spinBox, const QModelIndex &index) const
{
    bool ok;
    std::optional<int> limit = index.model()->data(index, Qt::UserRole).toInt(&ok);
    if (!ok)
        limit.reset();
    std::optional<int> val = index.model()->data(index, Qt::EditRole).toInt(&ok);
    if (!ok)
        val.reset();

    if(val && !limit)
        limit = val;

    QSignalBlocker blocker(spinBox);
    if (limit)
        spinBox->setMaximum(*limit);
    if (val)
        spinBox->setValue(*val);
    auto en = index.model()->flags(index) & Qt::ItemIsEnabled;
    spinBox->setEnabled(en);
}
