﻿#ifndef HISTMNGRWIDGET_H
#define HISTMNGRWIDGET_H

#include <QElapsedTimer>
#include <QMap>
#include <QMetaType>
#include <QWidget>
#include <QHash>

#include "HistMngrCfg.h"
#include "HistogramItem.h"
#include "DeviceCalibration.h"
#include "Histogramm.h"
#include "HistWorker.h"
#include "InlCheckResults.h"

namespace Ui {
class HistMngrWidget;
}

struct CompleteEvent;
class DeviceIndex;
struct DecodedData;
class QwtPlotCurve;
class QwtPlotMarker;
struct TdcChHit;
struct TqdcEvent;
class QTime;
class QTimer;
class QSemaphore;

struct InlStat
{
    QElapsedTimer time;
    QMap<int, double> count;
};

class HistMngrWidget : public QWidget
{
    Q_OBJECT
public:
    HistMngrWidget(QWidget *parent = nullptr);
    ~HistMngrWidget();
    static QVector<double> makeDNLCorrectionTable(const QVector<double> code_density);
    void updateInlCorr(QMap<DeviceIndex, DeviceCalibration> allCal);
    void updateInlCorr(const DeviceIndex &index, const QMap<int, QVector<double> > &corrMap);
    QHash<HistKey, Histogramm> getHists() const { return histogramms; }

signals:
    void inlCheckUpdated(InlCheckResults res);
    void histConfigChanged(const HistMngrCfg config);
    void clearHist(const HistKey &hist_key) const;

private slots:
    void refreshCurrentPlot(bool force=false);

    void on_comboBoxHistGroup_currentIndexChanged(int i);
    void on_checkBoxHistGroupEn_toggled(bool checked);
    void on_comboBoxCurDev_currentIndexChanged(int i);
    void on_spinBoxCurCh_valueChanged(int i);
    void on_comboBoxCurEdge_currentIndexChanged(int index);
    void on_groupBoxStopCh_toggled(bool val);
    void on_comboBoxStopDev_currentIndexChanged(int i);
    void on_spinBoxStopCh_valueChanged(int ch);
    void on_comboBoxStopEdge_currentIndexChanged(int index);
    void on_spinBoxMwMin_valueChanged(int min);
    void on_spinBoxMwMax_valueChanged(int max);
    void on_comboBoxHitTimeEdge_currentIndexChanged(int index);
    void on_radioButtonInl_toggled(bool checked);
    void on_radioButtonAdcAmplMin_toggled(bool checked);
    void on_pushButtonSave_clicked();
    void on_pushButtonSaveAll_clicked();
    void on_pushButtonClear_clicked();
    void on_pushButtonClearAll_clicked();
    void on_groupBoxGlobalEnable_toggled(bool checked);
    void on_groupBoxFit_toggled(bool checked);
    void on_checkBoxLogY_toggled(bool checked);
    void on_checkBoxZeroBin_clicked(bool checked);

public slots:
    void hist_updated(QHash<HistKey, Histogramm> histogramms_, bool forceRefresh=false);
    void inlCheckRequest();
    void devListUpdated(const DevChMap &tdcChMap,
                        const QHash<HistKey, Histogramm> &histogramms_);
    void setHistConfig(HistMngrCfg config_);
    void taiReperUpdated(HistKey hId, const TaiTime &taiReperSec);

private:
    void addBaseQwtItems(const HistGroupCfg &group_config);
    void addDeviceChQwtItems(HistId id, const DeviceIndex &index, int ch);
    void removeExtraQwtItems();
    void updateFit(const HistGroupCfg &histCfg, const Histogramm &hist) const;
    HistId getCurHistId() const;
    HistId getHistId(int index) const;
    DeviceIndex getDevIndex(bool getStopDev, int idInComboBox=-1) const;
    DeviceIndex getCurDev(bool getStopDev=false) const;
    int getCurCh() const;
    void onCurGroupChanged(); // update GUI for current hist & return histCfg
    void updateDeviceComboBoxItems(bool stopDev);
    void updateHistGroupComboBoxItems();
    QVector<double> getInl(const TqdcEvent &ev, const TdcChHit &tdcEv) const;
    QVector<double> getInl(const DeviceIndex &index, int ch) const;
    void updateHistToolTip();
    void updateSpinBoxMaxCh(bool stopDev);
    void block_gui_signals(bool block);
    void enable_hist_group(bool enable);
    QString getExportPath();

    DeviceIndexSet getDeviceIndexSet(const HistId &id) const;
    QSet<int> getDeviceChSet(const HistId &id, const DeviceIndex &index) const;

    Ui::HistMngrWidget *ui;
    QList<QWidget*> block_signal_widgets_list;
    int block_signal_counter = 0;
    QElapsedTimer lastRefreshTime;
    QTimer * const refreshTimer;
    InlResults inlFromConfig;
    InlStat inlStat;
    bool hide_zero_bin;

    HistMngrCfg config;
    DevChMap tdcChMap;

    QHash<HistKey, QwtPlotItem *> histQwtItems; // HistogramItem
    QHash<HistKey, QwtPlotItem *> histQwtCurve; // QwtPlotCurve
    QHash<HistKey, QVector<QwtPlotMarker *>> histMarkers;
    QHash<HistKey, Histogramm> histogramms;
    QHash<HistKey, QString> histToolTips; // memleak?
};

#endif // HISTMNGRWIDGET_H
