/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ZMQKVSUBSCRIBER_H
#define ZMQKVSUBSCRIBER_H

#include <zmq.hpp>
#include <QObject>
#include <QSet>
#include <QUrl>

#include "../AbstractSubscriber.h"

class ZmqKvSubscriber : public AbstractSubscriber
{
    Q_OBJECT
public:
    explicit ZmqKvSubscriber(const QString &serviceName, QObject *parent = nullptr);
    ~ZmqKvSubscriber();
    bool event(QEvent* ev) override;

public slots:
    void subscribe(const QString &topic) override;

private slots:
    void deferredInit();
    void pollEvents();

private:
    void createSocket();
    void deleteSocket();
    void restoreSubscription();
    void decodeMultipart(const std::vector<zmq::message_t> &msgs);

    zmq::context_t &context;
    zmq::socket_t *sock = nullptr;
    bool enabled = true;
    QUrl url;
    QSet<QString> topics;
};

#endif // ZMQKVSUBSCRIBER_H
