/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef REGOPSTREAM_H
#define REGOPSTREAM_H

#include "regop.h"

namespace mlink {

class RegOpStream
{
public:
    RegOpStream(const RegOpVector::const_iterator &first, const RegOpVector::const_iterator &last)
        : begin(first), end(last), rpos(begin) {}
    RegOpStream(const RegOpVector &v)
        : begin(v.begin()), end(v.end()), rpos(begin) {}
    RegOpStream(const std::optional<RegOpVector> &v)
    {
        if (v.has_value()) {
            begin = v->begin();
            end = v->end();
            rpos = begin;
        }
    }

    template<typename T>
    typename std::enable_if<std::is_integral<T>::value, RegOpStream &>::type
    operator>>(T &d) {
        // static_assert(std::is_integral<T>::value, "Integral type required");
        if (eof()) {
            valid = false; // read past end
            return *this;
        }
        if (eof()) {
            return *this;
        }
        auto &op = *(rpos++);
        if (op.result.ok)
            d = op.rdata;
        else
            valid = false;
        return *this;
    }

    bool eof() const { return rpos == end; }
    bool isValid() const { return valid; }

private:
    RegOpVector::const_iterator begin;
    RegOpVector::const_iterator end;
    RegOpVector::const_iterator rpos;
    bool valid = true;
};

template<typename T, std::size_t N>
RegOpStream &operator>>(RegOpStream &in, std::array<T, N> &d)
{
    for (auto &i : d)
        in >> i;
    return in;
}

template<typename T>
RegOpStream &operator>>(RegOpStream &in, std::vector<T> &d)
{
    for (auto &i : d)
        in >> i;
    return in;
}

template<typename T>
RegOpStream &operator>>(RegOpStream &in, QVector<T> &d)
{
    for (auto &i : d)
        in >> i;
    return in;
}

} // namespace mlink

using mlink::RegOpStream;

#endif // REGOPSTREAM_H
