#ifndef REGIO_REGIOBASE_H
#define REGIO_REGIOBASE_H

#include <cstdint>
#include <string>

#include <QDeadlineTimer>
#include <QHostAddress>
#include <QObject>

#include "regio/regiopacket.h"

namespace mlink {

class RegIOBase: public QObject
{
    Q_OBJECT

public:
    explicit RegIOBase(QObject *parent = nullptr) : QObject(parent) {}
    ~RegIOBase() override = default;
    void setDeleteFlag();
    bool hasDeleteFlag() const;
    void setHostAddress(const QHostAddress &address, int ttl);
    bool hasAddressExpired() const { return deviceAddressTimer.hasExpired(); }
    QHostAddress getHostAddress() const { return hasAddressExpired() ? QHostAddress() : deviceAddress; }
    virtual void reset();
    virtual void rawSend(const RegIoPacket &data) = 0; ///< \deprecated use sendPackets()
    virtual void sendPackets(const PacketList &);
    virtual size_t getCtrlPacketCount() const; ///< return number of control packets received
    bool getCtrlPacketBySeq(RegIoPacket *dest, int seq, double timeoutSec);
    [[nodiscard]] virtual PacketList getCtrlPackets(size_t packetCount, double timeout);
    virtual unsigned long getMaxDataSize() const = 0;

    std::string getDevName() const;
    unsigned long getDriverVersion() const;
    unsigned long getDriverRevision() const;

Q_SIGNALS:
    void rawDataReceived(const QByteArray &);

protected:
    virtual double getDeviceThroughput() const;
    bool packetTypeIsData(int type);
    virtual bool updateRxBuf(double timeoutMs) = 0;

    QHostAddress deviceAddress;
    QDeadlineTimer deviceAddressTimer;

    int bufSizeLimit = 0;
    PacketList rxBufCtrl; // TODO: hide
    std::atomic<bool> deleteFlag{false};
};

double getCurrentTimestamp();

} // namespace mlink

#endif /* REGIO_REGIOBASE_H */
