//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef REGIO_REGIOPACKET_H
#define REGIO_REGIOPACKET_H

#include <vector>
#include <deque>
#include <string>

#include <QtGlobal>
#include <QByteArray>
#include <QMetaType>
#include <QList>

class QTextStream;

namespace mlink {

using PacketRawDataType = quint32;
using PacketData = QVector<PacketRawDataType>;

class RegIoPacket {
public:
    RegIoPacket(quint16 _type = 0, quint16 _seq = 0, quint16 _src = 0, quint16 _dst = 0, const PacketData &_data = PacketData())
        : type(_type), seq(_seq), src(_src), dst(_dst), data(_data) { }
    void dump(const std::string &description = std::string()) const;
    quint16 type = 0;
    quint16 seq = 0;
    quint16 src = 0;
    quint16 dst = 0;
    PacketData data;
};

typedef QList<RegIoPacket> PacketList;

typedef std::vector<PacketRawDataType> PacketRawData;

} // namespace mlink

Q_DECLARE_METATYPE(mlink::RegIoPacket) /* has to be outside namespace */

std::ostream &operator<< (std::ostream & os, const mlink::RegIoPacket &p);
QTextStream &operator<< (QTextStream & os, const mlink::RegIoPacket &p);
QDebug operator<< (QDebug os, const mlink::RegIoPacket &p);
std::ostream &operator<< (std::ostream & os, const mlink::PacketRawData &p);

#endif // REGIO_REGIOPACKET_H
