#include "CMSubDialog.h"
#include "ui_CMSubDialog.h"

#include <QPushButton>

#include "base/ConfigSelector.h"

namespace {
auto ButtonDoTask = QDialogButtonBox::Ok;
auto ButtonDoTaskAndOpen = QDialogButtonBox::SaveAll;
}

CMSubDialog::CMSubDialog(const CMDialogType &_type, const QJsonObject &cfg_json,
                         const DaqModule &_program_type,
                         const ConfigIndexName &conf,
                         QWidget *parent)
    :
      QDialog(parent),
      ui(new Ui::CMSubDialog),
      type(_type),
      default_cfg_json(cfg_json),
      program_type(_program_type),
      configuration(conf),
      configuration_entered(configuration)
{
    ui->setupUi(this);

    if(type != CMDialog_Type_Rename){
        configuration_entered.getConfigurationName().append("(1)");
        ui->button_box_ok_cansel->button(ButtonDoTask)->setText(typeStr());
    }

    base_config.connectToDb();
    config_map = base_config.get_config_map(program_type);
    ui->button_box_ok_cansel->button(ButtonDoTaskAndOpen)->setText(typeStr() + " and open");
    setWindowTitle(type==CMDialog_Type_New ? "New default configuration"
                                  : typeStr()+" configuration");
    ui->line_edit_program_index->setText(configuration_entered.getIndex().getIndex());
    ui->line_edit_config_name->setText(configuration_entered.getConfigurationName());
    check_input_validity();
}

CMSubDialog::~CMSubDialog()
{
    delete ui;
}

ConfigSelector CMSubDialog::getConfigSelector() const
{
    return ConfigSelector(program_type, configuration_entered);
}

void CMSubDialog::on_line_edit_program_index_textEdited(const QString &arg1)
{
    configuration_entered.program_index = ProgramIndex::fromString(arg1.trimmed());
    check_input_validity();
}

void CMSubDialog::on_line_edit_config_name_textChanged(const QString &arg1)
{
    configuration_entered.configuration_name = arg1.trimmed();
    check_input_validity();
}

void CMSubDialog::check_input_validity()
{
    bool catch_up = configuration_entered.program_index.isEmpty() ||
            configuration_entered.configuration_name.isEmpty();

    if(type!=CMDialog_Type_Rename || configuration_entered != configuration) {
        catch_up = config_map.value(configuration_entered.program_index).contains(configuration_entered.configuration_name);
    }

    if(type != CMDialog_Type_Rename) {
        ui->button_box_ok_cansel->button(ButtonDoTask)->setEnabled(!catch_up);
    }

    ui->button_box_ok_cansel->button(ButtonDoTaskAndOpen)->setEnabled(!catch_up);

    QPalette palette;
    palette.setColor(QPalette::Base, catch_up ? QColor(240,128,128) : Qt::white);
    palette.setColor(QPalette::Text,Qt::black);
    ui->line_edit_program_index->setPalette(palette);
    ui->line_edit_config_name->setPalette(palette);
}

QString CMSubDialog::typeStr() const
{
    switch (type) {
    case CMDialog_Type_New: return "Create";
    case CMDialog_Type_Rename: return "Rename";
    case CMDialog_Type_Clone: return "Clone";
    default: return QString();
    }
}

void CMSubDialog::on_button_box_ok_cansel_clicked(QAbstractButton *button)
{
    auto stdButton = ui->button_box_ok_cansel->standardButton(button);

    if(stdButton != ButtonDoTaskAndOpen && stdButton != ButtonDoTask)
        return;

    if(type == CMDialog_Type_Rename) {
        if(configuration_entered == configuration) {
            return;
        }
        base_config.delete_config(ConfigSelector(program_type, configuration));
    }

    base_config.writeConfig(ConfigSelector(program_type, configuration_entered), default_cfg_json);
    if(stdButton == ButtonDoTaskAndOpen) {
        emit switch_to_config(configuration_entered);
    }
}
