/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MongoDbLogger.h"
#include <syslog.h>
#include <QDebug>
#include "base/SyslogMessageSender.h"

static int map_mongocxx_log_level(mongocxx::log_level level)
{
    switch (level) {
    case mongocxx::log_level::k_error: return LOG_CRIT;
    case mongocxx::log_level::k_critical: return LOG_ERR;
    case mongocxx::log_level::k_warning: return LOG_WARNING;
    case mongocxx::log_level::k_message: return LOG_NOTICE;
    case mongocxx::log_level::k_info: return LOG_INFO;
    case mongocxx::log_level::k_debug: return LOG_DEBUG;
    case mongocxx::log_level::k_trace: return LOG_DEBUG;
    }
    return LOG_WARNING;
}

void MongoDbLogger::operator()(mongocxx::log_level level,
                        mongocxx::stdx::string_view domain,
                        mongocxx::stdx::string_view message) noexcept
{
    auto prio = map_mongocxx_log_level(level);
    auto domain_str = std::string(domain);
    auto message_str = std::string(message);
    auto text = domain_str + ": " + message_str;
    SyslogMessageSender::log(prio, text.c_str());
}
