#ifndef REMOTECONTROLSERVER_H
#define REMOTECONTROLSERVER_H

#include <QObject>

#include "base/RcRunId.h"
#include "fsm/FsmState.h"
#include "RemoteControlServerStatus.h"
#include "remote-control-server/types.h"
#include "remote-control-server/RcProgramState.h"

class RemoteControlTlvServer;
struct RunDocument;
struct TypeLengthHeader;

class RemoteControlServer : public QObject
{
    Q_OBJECT
public:
    explicit RemoteControlServer(QObject *parent, quint16 preferedTcpPort=0, QString _descr="");
    virtual ~RemoteControlServer() override;

public slots:
    void setAllowRemoteControl(bool allow);
    __attribute__((deprecated("renamed to setAllowRemoteControl")))
    void startListen(bool state);
    void sendState(const QString &state);
    void onFsmStateChange(const FsmState &state);
    void onRcProgramStateChange(const RcProgramState &);
    void onRunError(const RcRunId &runId, QString runError=QString());
    void onRunWarning(const RcRunId &runId, const QString &warnStr);
    void onBackPressure(const RcRunId &runId, bool backPressure);
    void onVmeFlag(bool hasVme);
    void config_transfer(const RunDocument &rd) const;
    void evNumCheckResponse(int checkIndex, const EvNumSet &evNums) const;

signals:
    void startRequest(const RcRunId &runId, const QVariant &startEvNum);
    void stopRequest(const RcRunId &runId, const QVariant &stopEvNum);
    void gotRunDocument(const RunDocument &rd) const;
    void gotRcProgramState(const RcProgramState &) const; // all states from top RC
    void evNumCheckRequest(int checkIndex);
    void connectionStatusUpdated(const RemoteControlServerStatus &) const;
    void allowRemoteControlChanged(bool);

private slots:
    void connectionAccepted();
    void tlvDataReceived(const TypeLengthHeader &hdr, const QByteArray &data);

private:
    void sendFsmState() const;
    void sendRcProgramState() const;
    void send_json(const QJsonObject &obj) const;
    void command_line_parse(QByteArray data);
    void json_parse(const QJsonObject &obj);
    bool parseCommand(const QString &command, const QStringList &args);

    RemoteControlTlvServer * const server;
    FsmState fsmState;
    RcProgramState rcProgramState;
    RcRunId suppresedErrorRunId;
    QStringList unknownCommands;
};

#endif // REMOTECONTROLSERVER_H
